@@ -4,13 +4,13 @@
 
 """Tests for process behavior of coverage.py."""
 
-import distutils.sysconfig
 import glob
 import os
 import os.path
 import re
 import stat
 import sys
+import sysconfig
 import textwrap
 import time
 from xml.etree import ElementTree
@@ -23,7 +23,7 @@ from coverage.data import line_counts
 from coverage.files import python_reported_file
 from coverage.misc import output_encoding
 
-from tests.coveragetest import CoverageTest, TESTS_DIR
+from tests.coveragetest import CoverageTest, TESTS_DIR, xfail
 from tests.helpers import re_lines
 
 
@@ -742,12 +742,14 @@ class ProcessTest(CoverageTest):
         # about 5.
         self.assertGreater(line_counts(data)['os.py'], 50)
 
+    @xfail(
+        env.PYPY3 and env.PYPYVERSION >= (7, 1, 1),
+        "https://bitbucket.org/pypy/pypy/issues/3074"
+    )
     def test_lang_c(self):
         if env.JYTHON:
             # Jython as of 2.7.1rc3 won't compile a filename that isn't utf8.
             self.skipTest("Jython can't handle this test")
-        if env.PYPY3 and env.PYPYVERSION >= (7, 1, 1):          # pragma: obscure
-            self.xfail("https://bitbucket.org/pypy/pypy/issues/3074")
         # LANG=C forces getfilesystemencoding on Linux to 'ascii', which causes
         # failures with non-ascii file names. We don't want to make a real file
         # with strange characters, though, because that gets the test runners
@@ -1234,13 +1236,19 @@ class FailUnderTest(CoverageTest):
     def test_report_43_is_not_ok(self):
         st, out = self.run_command_status("coverage report --fail-under=44")
         self.assertEqual(st, 2)
-        self.assertEqual(self.last_line_squeezed(out), "forty_two_plus.py 7 4 43%")
+        self.assertEqual(
+            self.last_line_squeezed(out),
+            "Coverage failure: total of 43 is less than fail-under=44"
+        )
 
     def test_report_42p86_is_not_ok(self):
         self.make_file(".coveragerc", "[report]\nprecision = 2")
         st, out = self.run_command_status("coverage report --fail-under=42.88")
         self.assertEqual(st, 2)
-        self.assertEqual(self.last_line_squeezed(out), "forty_two_plus.py 7 4 42.86%")
+        self.assertEqual(
+            self.last_line_squeezed(out),
+            "Coverage failure: total of 42.86 is less than fail-under=42.88"
+        )
 
 
 class FailUnderNoFilesTest(CoverageTest):
@@ -1397,7 +1405,7 @@ def possible_pth_dirs():
 
     # If we're still looking, then try the Python library directory.
     # https://bitbucket.org/ned/coveragepy/issue/339/pth-test-malfunctions
-    yield distutils.sysconfig.get_python_lib()      # pragma: cant happen
+    yield sysconfig.get_python_lib()                # pragma: cant happen
 
 
 def find_writable_pth_directory():
