@@ -185,6 +185,13 @@ class XmlReportTest(XmlTestHelpers, CoverageTest):
         assert len(elts) == 1
         assert elts[0].get('line-rate') == '1'
 
+    def test_empty_file_is_skipped(self):
+        cov = self.run_doit()
+        cov.xml_report(skip_empty=True)
+        dom = ElementTree.parse("coverage.xml")
+        elts = dom.findall(".//class[@name='__init__.py']")
+        assert len(elts) == 0
+
     def test_curdir_source(self):
         # With no source= option, the XML report should explain that the source
         # is in the current directory.
@@ -370,6 +377,19 @@ class XmlPackageStructureTest(XmlTestHelpers, CoverageTest):
         dom = ElementTree.parse("coverage.xml")
         self.assert_source(dom, "src")
 
+    def test_relative_source(self):
+        self.make_file("src/mod.py", "print(17)")
+        cov = coverage.Coverage(source=["src"])
+        cov.set_option("run:relative_files", True)
+        self.start_import_stop(cov, "mod", modfile="src/mod.py")
+        cov.xml_report()
+
+        with open("coverage.xml") as x:
+            print(x.read())
+        dom = ElementTree.parse("coverage.xml")
+        elts = dom.findall(".//sources/source")
+        assert [elt.text for elt in elts] == ["src"]
+
 
 def compare_xml(expected, actual, **kwargs):
     """Specialized compare function for our XML files."""
